# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from msrest.pipeline import ClientRawResponse

from .. import models


class Header(object):
    """Header operations.

    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An objec model deserializer.
    """

    def __init__(self, client, config, serializer, deserializer):

        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer

        self.config = config

    def param_existing_key(
            self, user_agent, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header value "User-Agent": "overwrite"

        :param user_agent: Send a post request with header value
         "User-Agent": "overwrite"
        :type user_agent: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/existingkey'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['User-Agent'] = self._serialize.header("user_agent", user_agent, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_existing_key(
            self, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header value "User-Agent": "overwrite"

        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/existingkey'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'User-Agent': 'str',
            })
            return client_raw_response

    def param_protected_key(
            self, content_type, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header value "Content-Type": "text/html"

        :param content_type: Send a post request with header value
         "Content-Type": "text/html"
        :type content_type: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/protectedkey'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['Content-Type'] = self._serialize.header("content_type", content_type, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_protected_key(
            self, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header value "Content-Type": "text/html"

        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/protectedkey'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'Content-Type': 'str',
            })
            return client_raw_response

    def param_integer(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "positive",
        "value": 1 or "scenario": "negative", "value": -2

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param value: Send a post request with header values 1 or -2
        :type value: int
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/integer'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'int')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_integer(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header value "value": 1 or -2

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/integer'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'int',
            })
            return client_raw_response

    def param_long(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "positive",
        "value": 105 or "scenario": "negative", "value": -2

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param value: Send a post request with header values 105 or -2
        :type value: long
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/long'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'long')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_long(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header value "value": 105 or -2

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/long'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'long',
            })
            return client_raw_response

    def param_float(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "positive",
        "value": 0.07 or "scenario": "negative", "value": -3.0

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param value: Send a post request with header values 0.07 or -3.0
        :type value: float
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/float'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'float')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_float(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header value "value": 0.07 or -3.0

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/float'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'float',
            })
            return client_raw_response

    def param_double(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "positive",
        "value": 7e120 or "scenario": "negative", "value": -3.0

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param value: Send a post request with header values 7e120 or -3.0
        :type value: float
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/double'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'float')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_double(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header value "value": 7e120 or -3.0

        :param scenario: Send a post request with header values "scenario":
         "positive" or "negative"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/double'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'float',
            })
            return client_raw_response

    def param_bool(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "true", "value":
        true or "scenario": "false", "value": false

        :param scenario: Send a post request with header values "scenario":
         "true" or "false"
        :type scenario: str
        :param value: Send a post request with header values true or false
        :type value: bool
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/bool'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'bool')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_bool(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header value "value": true or false

        :param scenario: Send a post request with header values "scenario":
         "true" or "false"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/bool'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'bool',
            })
            return client_raw_response

    def param_string(
            self, scenario, value=None, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "valid", "value":
        "The quick brown fox jumps over the lazy dog" or "scenario": "null",
        "value": null or "scenario": "empty", "value": ""

        :param scenario: Send a post request with header values "scenario":
         "valid" or "null" or "empty"
        :type scenario: str
        :param value: Send a post request with header values "The quick brown
         fox jumps over the lazy dog" or null or ""
        :type value: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/string'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        if value is not None:
            header_parameters['value'] = self._serialize.header("value", value, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_string(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header values "The quick brown fox jumps over the
        lazy dog" or null or ""

        :param scenario: Send a post request with header values "scenario":
         "valid" or "null" or "empty"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/string'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'str',
            })
            return client_raw_response

    def param_date(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "valid", "value":
        "2010-01-01" or "scenario": "min", "value": "0001-01-01"

        :param scenario: Send a post request with header values "scenario":
         "valid" or "min"
        :type scenario: str
        :param value: Send a post request with header values "2010-01-01" or
         "0001-01-01"
        :type value: date
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/date'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'date')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_date(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header values "2010-01-01" or "0001-01-01"

        :param scenario: Send a post request with header values "scenario":
         "valid" or "min"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/date'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'date',
            })
            return client_raw_response

    def param_datetime(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "valid", "value":
        "2010-01-01T12:34:56Z" or "scenario": "min", "value":
        "0001-01-01T00:00:00Z"

        :param scenario: Send a post request with header values "scenario":
         "valid" or "min"
        :type scenario: str
        :param value: Send a post request with header values
         "2010-01-01T12:34:56Z" or "0001-01-01T00:00:00Z"
        :type value: datetime
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/datetime'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'iso-8601')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_datetime(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header values "2010-01-01T12:34:56Z" or
        "0001-01-01T00:00:00Z"

        :param scenario: Send a post request with header values "scenario":
         "valid" or "min"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/datetime'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'iso-8601',
            })
            return client_raw_response

    def param_datetime_rfc1123(
            self, scenario, value=None, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "valid", "value":
        "Wed, 01 Jan 2010 12:34:56 GMT" or "scenario": "min", "value": "Mon,
        01 Jan 0001 00:00:00 GMT"

        :param scenario: Send a post request with header values "scenario":
         "valid" or "min"
        :type scenario: str
        :param value: Send a post request with header values "Wed, 01 Jan
         2010 12:34:56 GMT" or "Mon, 01 Jan 0001 00:00:00 GMT"
        :type value: datetime
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/datetimerfc1123'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        if value is not None:
            header_parameters['value'] = self._serialize.header("value", value, 'rfc-1123')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_datetime_rfc1123(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header values "Wed, 01 Jan 2010 12:34:56 GMT" or
        "Mon, 01 Jan 0001 00:00:00 GMT"

        :param scenario: Send a post request with header values "scenario":
         "valid" or "min"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/datetimerfc1123'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'rfc-1123',
            })
            return client_raw_response

    def param_duration(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "valid", "value":
        "P123DT22H14M12.011S"

        :param scenario: Send a post request with header values "scenario":
         "valid"
        :type scenario: str
        :param value: Send a post request with header values
         "P123DT22H14M12.011S"
        :type value: timedelta
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/duration'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'duration')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_duration(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header values "P123DT22H14M12.011S"

        :param scenario: Send a post request with header values "scenario":
         "valid"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/duration'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'duration',
            })
            return client_raw_response

    def param_byte(
            self, scenario, value, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "valid", "value":
        "啊齄丂狛狜隣郎隣兀﨩"

        :param scenario: Send a post request with header values "scenario":
         "valid"
        :type scenario: str
        :param value: Send a post request with header values "啊齄丂狛狜隣郎隣兀﨩"
        :type value: bytearray
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/byte'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        header_parameters['value'] = self._serialize.header("value", value, 'bytearray')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_byte(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header values "啊齄丂狛狜隣郎隣兀﨩"

        :param scenario: Send a post request with header values "scenario":
         "valid"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/byte'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': 'bytearray',
            })
            return client_raw_response

    def param_enum(
            self, scenario, value=None, custom_headers={}, raw=False, **operation_config):
        """
        Send a post request with header values "scenario": "valid", "value":
        "GREY" or "scenario": "null", "value": null

        :param scenario: Send a post request with header values "scenario":
         "valid" or "null" or "empty"
        :type scenario: str
        :param value: Send a post request with header values 'GREY' .
         Possible values include: 'White', 'black', 'GREY'
        :type value: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/param/prim/enum'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')
        if value is not None:
            header_parameters['value'] = self._serialize.header("value", value, 'GreyscaleColors')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response

    def response_enum(
            self, scenario, custom_headers={}, raw=False, **operation_config):
        """
        Get a response with header values "GREY" or null

        :param scenario: Send a post request with header values "scenario":
         "valid" or "null" or "empty"
        :type scenario: str
        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/response/prim/enum'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)
        header_parameters['scenario'] = self._serialize.header("scenario", scenario, 'str')

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            client_raw_response.add_headers({
                'value': models.GreyscaleColors,
            })
            return client_raw_response

    def custom_request_id(
            self, custom_headers={}, raw=False, **operation_config):
        """
        Send x-ms-client-request-id = 9C4D50EE-2D56-4CD3-8152-34347DC9F2B0 in
        the header of the request

        :param dict custom_headers: headers that will be added to the request
        :param boolean raw: returns the direct response alongside the
         deserialized response
        :rtype: None
        :rtype: msrest.pipeline.ClientRawResponse if raw=True
        """
        # Construct URL
        url = '/header/custom/x-ms-client-request-id/9C4D50EE-2D56-4CD3-8152-34347DC9F2B0'

        # Construct parameters
        query_parameters = {}

        # Construct headers
        header_parameters = {}
        header_parameters['Content-Type'] = 'application/json; charset=utf-8'
        if custom_headers:
            header_parameters.update(custom_headers)

        # Construct and send request
        request = self._client.post(url, query_parameters)
        response = self._client.send(request, header_parameters, **operation_config)

        if response.status_code not in [200]:
            raise models.ErrorException(self._deserialize, response)

        if raw:
            client_raw_response = ClientRawResponse(None, response)
            return client_raw_response
